/*
UNIVERSITE PARIS 1 (PANTHEON SORBONNE)
MIAGE - UFR 27
Cours ISI5 / INF2 - Developpement d'Interface

 */
package ihmexamples.volumeapplication;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * Example d'usage du modele MVC
 * Cette classe definit une vue basé sur un "Spinner" (textfield contenant
 * des boutons pour augmenter/reduire la valeur) pour visualiser et
 * modifier la valeur du modele de volume
 *
 * Example base sur Baptiste Wicht, <b>"Implémentation du pattern MVC"</b>,
 * 24 Avril 2007 (derniere visite le 29 Janvier 2009).
 * @see http://baptiste-wicht.developpez.com/tutoriel/conception/mvc/
 * @author kirsch
 */
public class VolumeSpinnerView extends VolumeView {

    private JFrame jFrame;
    private SpinnerNumberModel spinnerModel;
    private JSpinner jSpinner;

    public VolumeSpinnerView(VolumeController controller) {
        this(controller, 0);
    }

    public VolumeSpinnerView(VolumeController controller, int volume) {
        super(controller);
        this.buildFrame(volume);
    }

    private void buildFrame(int volume) {
        this.jFrame = new JFrame();
        jFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        /* on definit le layoutmanager avec 2x plus d'espace vert. et horiz. */
        BorderLayout border = new BorderLayout();
        border.setHgap(border.getHgap() * 2);
        border.setVgap(border.getVgap() * 2);
        jFrame.setLayout(border);

        //SpinnerNumberModel (val, min, max, step);
        spinnerModel = new SpinnerNumberModel(volume, 0, 100, 1);
        jSpinner = new JSpinner(spinnerModel);

        JButton jButton = new JButton("Change");
        jButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                // notifyVolumeChange(e); //solution alternative
                notifyVolumeChange();
            }
        });

        jFrame.add(jSpinner, BorderLayout.CENTER);
        jFrame.add(jButton, BorderLayout.SOUTH);

        jFrame.pack();
    }

    // solution alternative
    /* private void notifyVolumeChange(ChangeEvent e) {
    JSpinner spinner = (JSpinner)e.getSource();
    int nv = spinner.getModel().getNumber().intValue();
    this.getController().notifyVolumeChange(nv);
    }
     */
    private void notifyVolumeChange() {
        int newVolume = this.spinnerModel.getNumber().intValue();
        this.getController().notifyVolumeChange(newVolume);
    }

    public void volumeChanged(VolumeChangedEvent event) {
        spinnerModel.setValue(event.getNewVolume());
    }

    @Override
    public void display() {
        this.jFrame.setVisible(true);
    }

    @Override
    public void close() {
        this.jFrame.setVisible(false);
    }

    @Override
    public void setLocation(int x, int y) {
        this.jFrame.setLocation(x, y);
    }

    @Override
    public void setSize(Dimension d) {
        this.jFrame.setSize(d);
    }

    @Override
    public Dimension getSize() {
        return this.jFrame.getSize();
    }
}
